﻿if myBigIgnite == nil then
	return
end


local mBI = myBigIgnite
local L = LibStub("AceLocale-3.0"):GetLocale("myBigIgnite", true)
local LSM = LibStub("LibSharedMedia-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")
mBI.cfMode = false

local pairs, type, tonumber, tostring, print, tinsert, wipe = pairs, type, tonumber, tostring, print, tinsert, wipe
local _


-------------------------------------------------------------------------------------------------------------------------------------------
-- House keeping functions

local function deepCopy(src)
	local res = {}
    for k, v in pairs(src) do
        if type(v) == "table" then
            v = deepCopy(v)
        end
        res[k] = v
    end
    return res
end

local function selectCopy(t, ref)
	local tab = {}
	if t ~= nil then
		for k, v in pairs(ref) do
			if t[k] == nil then
				if type(v) == "table" then
					v = deepCopy(v)
				end
				tab[k] = v
			else
				if type(t[k]) == "table" then
					t[k] = selectCopy(t[k], v)
				end
				tab[k] = t[k]
			end
		end
	else
		tab = deepCopy(ref)
	end
	return tab	
end

local function isPositiveNum(info, value)
	if tonumber(value) ~= nil and tonumber(value)> 0 then
		return true
	else
		print(mBI.cD .. L["myBigIgnite: "] .. L["Enter a positive number."])
	end
end

local function isIcone()
	if mBI.db.profile.iconFrame.enabled then
		if mBI.db.profile.iconFrame.type == "normal" or mBI.db.profile.iconFrame.type == "zoomed" then
			return true
		else
			return false
		end
	else
		return true
	end
end

local function isInCombat()
	return mBI.inCombat
end

local function round(number, dec)
	return math.floor(number * 10^dec + 0.5) / 10^dec
end

local function formatText(txt)
	if type(txt) == "number" then
		if mBI.db.profile.mainFrame.textFormat == "condensed" then
			txt = round(txt / 1000, 1) .. "k"
		end
	end
	return txt
end

local function getColor(t)
	return t.r, t.g, t.b, t.a
end

local function setColor(t, r, g, b, a)
	t.r, t.g, t.b, t.a = r, g, b, a or t.a
end

local function setLink(t1, t2, value)
	if value then
		t2.r, t2.g, t2.b = t1.r, t1.g, t1.b
	end	
end

function myBigIgnite:List(mediatype)
	local t = {}
	for k, v in pairs(LSM:List(mediatype)) do
		t[v] = v
	end
	return t
end


-------------------------------------------------------------------------------------------------------------------------------------------
-- Option tables

function myBigIgnite:LoadFakeOptions()																		-- default option table
	mBI.fakeOptions = {
		type = "group",
		name = "myBigIgnite",
		childGroups = "tab",
		disabled = isInCombat,
		args = {
			toggleOptions = {
				type = "execute",
				name = L["myBigIgnite config"],
				desc = L["Toggles myBigIgnite configuration panel."],
				order = 1,
				func = function()
					self:ChangeConfigDisplay("fireSpec")
					mBI.configLoaded = true
				end,
			},
		},
	}

	mBI.options = deepCopy(mBI.fakeOptions)																	-- ACE registry
	ACR:RegisterOptionsTable("myBigIgnite", mBI.options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("myBigIgnite", "myBigIgnite")
end	

function myBigIgnite:LoadooFOptions()																		-- config panel, switching out of fire spec
	mBI.ooFOptions = {
		type = "group",
		name = "myBigIgnite",
		childGroups = "tab",
		args = {
			message = {
				type = "description",
				name = L["Configuration available only when myBigIgnite is enabled."],
				fontSize = "medium",
			},
		},
	}
end

function myBigIgnite:LoadFireOptions()																		-- real config panel
	local UIScale = UIParent:GetScale()
	local UIWidth = UIParent:GetWidth()
	local UIHeight = UIParent:GetHeight()
	mBI.fireOptions = {
		type = "group",
		name = "myBigIgnite",
		childGroups = "tab",
		disabled = isInCombat,
		args = {
			configMode = {
				type = "toggle",
				name = L["Configuration mode"],
				desc = L["Enable myBigIgnite configuration mode."],
				order = 1,
				width = "double",
				get = function()
					return mBI.cfMode
				end,
				set = function(info, value)
					mBI.cfMode = value
					myBigIgnite:ConfigMode(value)
				end,
			},
			reset = {
				type = "execute",
				name = L["Reset"],
				desc = L["Reset all settings."],
				order = 2,
				confirm = true,
				confirmText = L["This will reset all myBigIgnite settings. Are you sure?"],
				func = function()
					for k, v in pairs(mBI.db.profile) do
						if k ~= "locale" then
							wipe(mBI.db.profile[k])
							mBI.db.profile[k] = deepCopy(mBI.defaults.profile[k])
						end
					end
					self:RefreshConfig()
					print(mBI.cL .. L["myBigIgnite has been reset."])
				end,
			},
			tabTemplates = {
				type = "group",
				name = L["Templates"],
				desc = L["Quick layout setting."],
				order = 3,
				args = {
				},
			},
			tabWindow = {
				type = "group",
				name = L["Layout"],
				desc = L["Layout options."],
				order = 4,
				args = {
					funcHeader = {
						type = "header",
						name = L["Autohiding and optional frames"],
						order = 1,
					},
					autohideDesc = {
						type = "description",
						name = mBI.cC .. L["Hide automatically myBigIgnite if:"],
						fontSize = "medium",
						width = "full",
						order = 2,
					},
					autoHideCombat = {
						type = "toggle",
						name = L["Out of combat"],
						order = 3,
						get = function()
							return mBI.db.profile.mainFrame.hiding.noCombat
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.hiding.noCombat = value
							if not mBI.cfMode then
								self:FrameFading(value)
							end
							if not value then
								mBI.db.profile.mainFrame.hiding.noIgnite = false
								mBI.db.profile.mainFrame.hiding.combuCD = false
							end
						end,
					},
					autoHideIgnite = {
						type = "toggle",
						name = L["No Ignite"],
						order = 4,
						get = function()
							return mBI.db.profile.mainFrame.hiding.noIgnite
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.hiding.noIgnite = value
							if value then
								mBI.db.profile.mainFrame.hiding.noCombat = true
								if not mBI.cfMode then
									self:FrameFading(value)
								end
							end
						end,
					},
					space1 = {
						type = "description",
						name = " ",
						width = "full",
						order = 5,
					},
					autoHideCombu = {
						type = "toggle",
						name = L["Combustion in CD"],
						order = 6,
						disabled = function()
							return not mBI.db.profile.settings.combuTrack
						end,
						get = function()
							return mBI.db.profile.mainFrame.hiding.combuCD
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.hiding.combuCD = value
							if value then
								mBI.db.profile.mainFrame.hiding.noCombat = true
								if not mBI.cfMode then
									self:FrameFading(value)
								end
							end
						end,
					},
					autoHideCombuTime = {
						type = "range",
						name = L["Time before CD end"],
						desc = L["Shows the frame when Combustion CD is less than this value."],
						order = 7,
						disabled = function()
							return not mBI.db.profile.settings.combuTrack or not mBI.db.profile.mainFrame.hiding.combuCD
						end,
						min = 0, max = 45, softMax = 30, step = 1,
						get = function()
							return mBI.db.profile.mainFrame.hiding.combuCDTime
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.hiding.combuCDTime = value
						end,
					},
					space2 = {
						type = "description",
						name = " ",
						width = "full",
						order = 8,
					},
					optionalDesc = {
						type = "description",
						name = "|cffFFC000" .. L["Enable optional frames:"],
						fontSize = "medium",
						width = "full",
						order = 9,
					},
					isIcon = {
						type = "toggle",
						name = L["Icon"],
						desc = L["Enables display of the Ignite icon."],
						order = 10,
						get = function()
							return mBI.db.profile.iconFrame.enabled
						end,
						set = function(info, value)
							mBI.db.profile.iconFrame.enabled = value
							if mBI.db.profile.iconFrame.enabled and not mBI.F.icon then
								self:CreateIconFrame()
							end
							if mBI.db.profile.iconFrame.enabled then
								self:SetFrameSize("icon")
								self:SetFramePoint("icon", mBI.db.profile.iconFrame.side, mBI.F.main, mBI.db.profile.iconFrame.side, mBI.F.icon.off, 0)
								mBI.F.icon:Show()
							else
								mBI.F.icon:Hide()
								mBI.db.profile.iconFrame.impactIcon = false
								if not mBI.db.profile.values.impactPerctEnabled then
									mBI.db.profile.settings.impactMode = false
								end
							end
							self:SetFrameSize("icon")
							self:SetFrameSize("main")
							self:SetFramePoint("icon", mBI.db.profile.iconFrame.side, mBI.F.main, mBI.db.profile.iconFrame.side, mBI.F.icon.off, 0)
							self:SetFramePoint("dmg", mBI.db.profile.mainFrame.textAlign, mBI.F.main, mBI.db.profile.mainFrame.textAlign, 0, 0)
							self:AlignText("dmg")
							if mBI.db.profile.titleFrame.enabled then
								self:SetFramePoint("title", mBI.db.profile.mainFrame.textAlign, mBI.F.main, mBI.db.profile.mainFrame.textAlign, 0, 0)
								self:AlignText("title")
							end
						end,
					},
					isTitle = {
						type = "toggle",
						name = L["Title"],
						desc = L["Enables display of \"Ignite\" on the top of the frame."],
						order = 11,
						get = function()
							return mBI.db.profile.titleFrame.enabled
						end,
						set = function(info, value)
							mBI.db.profile.titleFrame.enabled = value
							if mBI.db.profile.titleFrame.enabled and not mBI.F.title then
								self:CreateTitleFrame()
							end
							if mBI.db.profile.titleFrame.enabled then
								self:SetFrameSize("title")
								self:SetFramePoint("title", mBI.db.profile.mainFrame.textAlign, mBI.F.main, mBI.db.profile.mainFrame.textAlign, 0, 0)
								mBI.F.title:Show()
							else
								mBI.F.title:Hide()
							end
							if mBI.db.profile.iconFrame.enabled then
								self:SetFrameSize("icon")
							end
							self:SetFrameSize("main")
							self:UpdateDisplay("low")
						end,
					},
					-- isHUD = {
						-- type = "toggle",
						-- name = L["HUD"],
						-- desc = L["Enables Power Aura style notification."],
						-- order = 10,
						-- disabled = true,
						-- get = function()
							-- return mBI.db.profile.hudFrame.enabled
						-- end,
						-- set = function(info, value)
							-- mBI.db.profile.hudFrame.enabled = value
						-- end,
					-- },
					space3 = {
						type = "description",
						name = " ",
						width = "full",
						order = 12,
					},
					mainDispHeader = {
						type = "header",
						name = L["Main window"],
						order = 13,
					},
					space11 = {
						type = "description",
						name = " ",
						width = "full",
						order = 14,
					},					
					xPosition = {
						type = "range",
						name = L["Horizontal position"],
						order = 15,
						min = 0,
						max = round(UIWidth, 0),
						softMin = 0,
						softMax = round(UIWidth, 0),
						bigStep = 0.5,
						get = function()
							return mBI.db.profile.anchorFrame.coord.xcd / UIScale
						end,
						set = function(info, value)
							mBI.db.profile.anchorFrame.coord.xcd = value * UIScale
							self:SetAnchorFramePosition()
						end,
					},
					yPosition = {
						type = "range",
						name = L["Vertical position"],
						order = 16,
						min = 0,
						max = round(UIHeight, 0),
						softMin = 0,
						softMax = round(UIHeight, 0),
						bigStep = 0.5,
						get = function()
							return mBI.db.profile.anchorFrame.coord.ycd / UIScale
						end,
						set = function(info, value)
							mBI.db.profile.anchorFrame.coord.ycd = value * UIScale
							self:SetAnchorFramePosition()
						end,
					},
					resetPosition = {
						type = "execute",
						name = L["Reset position"],
						desc = L["Reset the main frame to its default position."],
						order = 17,
						confirm = true,
						confirmText = L["Reset the main frame position?"],
						func = function()
							mBI.db.profile.anchorFrame.coord = deepCopy(mBI.defaults.profile.anchorFrame.coord)
							self:SetAnchorFramePosition()
						end,
					},					
					space12 = {
						type = "description",
						name = " ",
						width = "full",
						order = 18,
					},
					textBase = {
						type = "range",
						name = L["Text and window size"],
						order = 19,
						min = 6,
						max = 20,
						softMin = 8,
						softMax = 18,
						step = 1,
						bigStep = 1,
						get = function()
							return mBI.db.profile.mainFrame.textSize
						end,
						set = function (info, value)
							mBI.db.profile.mainFrame.textSize = value
							self:SetFrameSize("dmg")
							if mBI.F.igniteTimer then
								self:SetIgniteTimer(mBI.F.igniteTimer)
							end
							if mBI.db.profile.titleFrame.enabled then
								self:SetFrameSize("title")
							end
							if mBI.db.profile.iconFrame.enabled then
								self:SetFrameSize("icon")
								self:SetFramePoint("icon", mBI.db.profile.iconFrame.side, mBI.F.main, mBI.db.profile.iconFrame.side, mBI.F.icon.off, 0)
							end
							self:SetFrameSize("main")
						end,
					},
					mainFrameAlign = {
						type = "select",
						name = L["Main Window alignment"],
						order = 20,
						values = {
							RIGHT = L["Right"],
							LEFT = L["Left"],
							CENTER = L["Center"],
							TOP = L["Top"],
							BOTTOM = L["Bottom"],
							TOPLEFT = L["TopLeft"],
							BOTTOMLEFT = L["BottomLeft"],
							TOPRIGHT = L["TopRight"],
							BOTTOMRIGHT = L["BottomRight"],
						},
						get = function()
							return mBI.db.profile.mainFrame.align
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.align = value
							self:SetFramePoint("main", mBI.db.profile.mainFrame.align, mBI.F.anchor, "CENTER")
						end,
					},
					anchorStrata = {
						type = "select",
						name = L["Window strata"],
						order = 21,
						values = {
								[1] = L["Low"],
								[2] = L["Medium"],
								[3] = L["High"],	
						},
						get = function()
							return mBI.db.profile.anchorFrame.strata
						end,
						set = function(info, value)
							mBI.db.profile.anchorFrame.strata = value							
							self:SetAnchorFrameStrata()
						end,
					},
					space13 = {
						type = "description",
						name = " ",
						width = "full",
						order = 22,
					},
					enableBg = {
						type = "toggle",
						name = L["Enable background"],
						desc = L["Enables main window background."],
						order = 23,
						get = function()
							return mBI.db.profile.mainFrame.bgEnabled
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.bgEnabled = value
							self:SetFrameBackdrop("main")
							self:UpdateDisplay("low")
						end,
					},
					bgFile = {
						type = "select",
						name = L["Background"],
						desc = L["Selects the main window background."],
						order = 24,
						width = "double",
						disabled = function()
							return not mBI.db.profile.mainFrame.bgEnabled
						end,
						values = function()
							return self:List("background")
						end,
						get = function()
							return mBI.db.profile.mainFrame.bgTexture
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.bgTexture = value
							self:SetFrameBackdrop("main")
							self:UpdateDisplay("low")
						end,
					},
					space14 = {
						type = "description",
						name = " ",
						width = "full",
						order = 25,
					},					
					enableBd = {
						type = "toggle",
						name = L["Enable border"],
						desc = L["Enables main window border."],
						order = 26,
						get = function()
							return mBI.db.profile.mainFrame.bdEnabled
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.bdEnabled = value
							self:SetFrameBackdrop("main")
							self:UpdateDisplay("low")
						end,
					},					
					bdFile = {
						type = "select",
						name = L["Border"],
						desc = L["Selects the main window border."],
						order = 27,
						width = "double",
						disabled = function()
							return not mBI.db.profile.mainFrame.bdEnabled
						end,
						values = function()
							return self:List("border")
						end,
						get = function()
							return mBI.db.profile.mainFrame.bdTexture
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.bdTexture = value
							self:SetFrameBackdrop("main")
							self:UpdateDisplay("low")
						end,
					},
					bgInset = {
						type = "range",
						name = L["Background inset"],
						order = 28,
						disabled = function()
							return not mBI.db.profile.mainFrame.bgEnabled
						end,
						min = -10,
						max = 10,
						softMin = -10,
						softMax = 10,
						step = 0.05,
						bigStep = 0.25,
						get = function()
							return mBI.db.profile.mainFrame.inset
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.inset = value						
							self:SetFrameBackdrop("main")
							self:UpdateDisplay("low")
						end,
					},
					bdThickness = {
						type = "range",
						name = L["Border thickness"],
						order = 29,
						disabled = function()
							return not mBI.db.profile.mainFrame.bdEnabled
						end,
						min = 0,
						max = 35,
						softMin = 0.5,
						softMax = 30,
						step = 0.05 ,
						bigStep = 0.25,
						get = function()
							return mBI.db.profile.mainFrame.bdSize
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.bdSize = value
							self:SetFrameBackdrop("main")
							self:UpdateDisplay("low")
						end,
					},
					space15 = {
						type = "description",
						name = " ",
						width = "full",
						order = 30,
					},
					textHeader = {
						type = "header",
						name = L["Text"],
						order = 31,
					},
					fontFile = {
						type = "select",
						name = L["Font"],
						order = 32,
						values = function()
							return self:List("font")
						end,
						get = function()
							return mBI.db.profile.mainFrame.textFont
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.textFont = value
							mBI.font = LSM:Fetch("font", mBI.db.profile.mainFrame.textFont)
							mBI.F.dmg.txt:SetFont(mBI.font, mBI.db.profile.mainFrame.textSize, mBI.db.profile.mainFrame.lowIg.textFlag)
							if mBI.db.profile.titleFrame.enabled then
								mBI.F.title.txt:SetFont(mBI.font, mBI.db.profile.mainFrame.textSize, mBI.db.profile.mainFrame.lowIg.textFlag)
								self:SetFrameSize("title")
							end
							self:SetFrameSize("dmg")
							if mBI.db.profile.iconFrame.enabled then
								self:SetFrameSize("icon")
							end
							self:SetFrameSize("main")
							if mBI.F.igniteTimer then
								self:SetIgniteTimer(mBI.F.igniteTimer)
							end
						end,
					},
					textFormat = {
						type = "select",
						name = L["Text format"],
						order = 33,
						values = {
							["normal"] = L["Normal"],
							["condensed"] = L["Condensed"],
						},
						get = function()
							return mBI.db.profile.mainFrame.textFormat
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.textFormat = value
							self:SetFrameSize("dmg")
							if mBI.db.profile.iconFrame.enabled then
								self:SetFrameSize("icon")
							end
							self:SetFrameSize("main")
							if mBI.F.igniteTimer then
								self:SetIgniteTimer(mBI.F.igniteTimer)
							end
						end,						
					},
					textShadow = {
						type = "toggle",
						name = L["Text shadow"],
						order = 34,
						get = function()
							return mBI.db.profile.mainFrame.textShadow
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.textShadow = value
							self:SetFrameSize("dmg")
							if mBI.db.profile.titleFrame.enabled then
								self:SetFrameSize("title")
							end
						end,
					},
					space16 = {
						type = "description",
						name = " ",
						width = "full",
						order = 35,
					},					
					textAlign = {
						type = "select",
						name = L["Text alignement"],
						desc = L["Disabled if Ignite icon is displayed."],
						order = 36,
						disabled = function()
							return mBI.db.profile.iconFrame.enabled
						end,
						values = {
							RIGHT = L["Right"],
							LEFT = L["Left"],
							CENTER = L["Center"],
						},
						get = function()
							return mBI.db.profile.mainFrame.textAlign
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.textAlign = value
							self:AlignText("dmg")
							self:SetFramePoint("dmg", mBI.db.profile.mainFrame.textAlign, mBI.F.main, mBI.db.profile.mainFrame.textAlign, 0, 0)
							if mBI.db.profile.titleFrame.enabled then
								self:AlignText("title")
								self:SetFramePoint("title", mBI.db.profile.mainFrame.textAlign, mBI.F.main, mBI.db.profile.mainFrame.textAlign, 0, 0)
							end
						end,
					},
					textOffsetX = {
						type = "range",
						name = L["Horizontal offset"],
						order = 37,
						disabled = function()
							if not mBI.db.profile.iconFrame.enabled then
								if mBI.db.profile.mainFrame.textAlign == "CENTER" then
									return true
								else
									return false
								end
							else
								return false
							end
						end,
						min = 0,
						max = 10,
						softMin = 0,
						softMax = 10,
						step = 0.1,
						bigStep = 0.25,
						get = function()
							return mBI.db.profile.mainFrame.textOffsetX
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.textOffsetX = value
							self:AlignText("dmg")
							if mBI.db.profile.titleFrame.enabled then
								self:AlignText("title")
							end
						end,
					},
					textOffsetY = {
						type = "range",
						name = L["Vertical offset"],
						order = 38,
						disabled = function()
							if not mBI.db.profile.iconFrame.enabled then
								return false
							else
								return false
							end
						end,
						min = -2.5,
						max = 2.5,
						softMin = -2.5,
						softMax = 2.5,
						step = 0.1,
						bigStep = 0.25,
						get = function()
							return mBI.db.profile.mainFrame.textOffsetY
						end,
						set = function(info, value)
							mBI.db.profile.mainFrame.textOffsetY = value
							self:AlignText("dmg")
							if mBI.db.profile.titleFrame.enabled then
								self:AlignText("title")
							end
						end,
					},
					space17 = {
						type = "description",
						name = " ",
						width = "full",
						order = 39,
					},
					iconHeader = {
						type = "header",
						name = L["Icon"],
						order = 40,
					},										
					iconeSize = {
						type = "range",
						name = L["Icon size"],
						order = 41,
						disabled = function()
							return not mBI.db.profile.iconFrame.enabled
						end,
						isPercent = true,
						min = 0.5,
						max = 1.5,
						softMin = 0.6,
						softMax = 1.45,
						step = 0.01,
						bigStep = 0.05,					
						get = function()
							return mBI.db.profile.iconFrame.sizeFactor
						end,
						set = function(info, value)
							mBI.db.profile.iconFrame.sizeFactor = value
							self:SetFrameSize("icon")
							self:SetFrameSize("main")
							self:SetFramePoint("icon", mBI.db.profile.iconFrame.side, mBI.F.main, mBI.db.profile.iconFrame.side, mBI.F.icon.off, 0)
						end,
					},					
					iconSide = {
						type = "select",
						name = L["Icon side"],
						order = 42,
						disabled = function()
							return not mBI.db.profile.iconFrame.enabled
						end,
						values = {
							RIGHT = L["Right"],
							LEFT = L["Left"],
						},
						get = function()
							return mBI.db.profile.iconFrame.side
						end,
						set = function(info, value)
							mBI.db.profile.iconFrame.side = value
							self:SetFramePoint("icon", mBI.db.profile.iconFrame.side, mBI.F.main, mBI.db.profile.iconFrame.side, mBI.F.icon.off, 0)
							self:SetFramePoint("dmg", mBI.db.profile.mainFrame.textAlign, mBI.F.main, mBI.db.profile.mainFrame.textAlign, 0, 0)
							self:AlignText("dmg")
							if mBI.db.profile.titleFrame.enabled then
								self:SetFramePoint("title", mBI.db.profile.mainFrame.textAlign, mBI.F.main, mBI.db.profile.mainFrame.textAlign, 0, 0)
								self:AlignText("title")
							end
						end,
					},
					iconeType = {
						type = "select",
						name = L["Icon type"],
						desc = L["Changes Ignite icon skin."],
						order = 43,
						disabled = function()
							return not mBI.db.profile.iconFrame.enabled
						end,
						values = {
							normal = L["Normal"],
							zoomed = L["Zoomed"],
							normbord = L["Normal with border"],
							zoombord = L["Zoomed with border"],
						},
						get = function()
							return mBI.db.profile.iconFrame.type
						end,
						set = function(info, value)
							mBI.db.profile.iconFrame.type = value
							self:SetIconType("icon", mBI.db.profile.iconFrame.type)
							self:SetFrameBackdrop("icon")
							self:SetIconBorderColor(mBI.db.profile.mainFrame.lowIg.bdColor)
						end,
					},
					bdIconThickness = {
						type = "range",
						name = L["Icon border thickness"],
						order = 44,
						disabled = isIcone,
						min = 0.2,
						max = 5,
						softMin = 0.25,
						softMax = 5,
						step = 0.1,
						bigStep = 0.25,
						get = function()
							return mBI.db.profile.iconFrame.bdSize
						end,
						set = function(info, value)
							mBI.db.profile.iconFrame.bdSize = value
							self:SetFrameBackdrop("icon")
							self:SetIconBorderColor(mBI.db.profile.mainFrame.lowIg.bdColor)
						end,
					},
				},
			},
			tabSpells = {
				type = "group",
				name = L["Spells"],
				desc = L["Spells to monitor."],
				order = 5,
				args = {
					auraHeader = {
						type = "header",
						name = L["DoT tracking"],
						order = 1,
					},
					livingBomb = {
						type = "toggle",
						name = L["livingBomb"],
						desc = L["Toggle to activate Living Bomb dot tracking."],
						order = 2,
						width = "full",
						get = function ()
							return mBI.db.profile.spellTracking.livingBomb.enabled
						end,
						set = function (info, value)
							mBI.db.profile.spellTracking.livingBomb.enabled = value
							self:CreateSpellTables()
						end,
					},
					pyroblast = {
						type = "toggle",
						name = L["pyroblast"],
						desc = L["Toggle to activate Pyroblast dot tracking."],
						order = 3,
						width = "full",
						get = function ()
							return mBI.db.profile.spellTracking.pyroblast.enabled
						end,
						set = function (info, value)
							mBI.db.profile.spellTracking.pyroblast.enabled = value
							self:CreateSpellTables()
						end,
					},
					ffbolt = {
						type = "toggle",
						name = L["ffbolt"],
						order = 4,
						desc = L["Toggle to activate Frostfire Bolt dots tracking."],
						width = "full",
						get = function ()
							return mBI.db.profile.spellTracking.ffbolt.enabled
						end,
						set = function (info, value)
							mBI.db.profile.spellTracking.ffbolt.enabled = value
							self:CreateSpellTables()
						end,
					},
					imp_combuHeader = {
						type = "header",
						name = L["Combustion and Impact tracking"],
						order = 10,
					},
					combuTracking = {
						type = "toggle",
						name = L["Combustion cooldown"],
						order = 11,
						desc = L["Toggle to activate Combustion cooldown tracking."],
						width = "full",
						get = function()
							return mBI.db.profile.settings.combuTrack
						end,
						set = function(info, value)
							mBI.db.profile.settings.combuTrack = value
							if not value then
								mBI.db.profile.settings.impactMode = false
								mBI.combuCD = false
								mBI.combuCDTimer = 0
							else
								self:CombuCheck()
							end
						end,
					},
					impactTracking= {
						type = "toggle",
						name = L["Impact mode"],
						order = 12,
						desc = L["Toggle to activate Impact mode."],
						disabled = function()
							return not mBI.db.profile.settings.combuTrack
						end,
						get = function()
							return mBI.db.profile.settings.impactMode
						end,
						set = function(info, value)
							mBI.db.profile.settings.impactMode = value
							mBI.db.profile.values.impactPerctEnabled = value
							if mBI.db.profile.iconFrame.enabled and mBI.db.profile.iconFrame.dynamic then
								mBI.db.profile.iconFrame.impactIcon = value
							end
						end,
					},
					impactPerctEnable = {
						type = "toggle",
						name = L["Threshold"],
						desc = L["Enables threshold for Impact notification."],
						order = 13,
						disabled = function()
							return not mBI.db.profile.settings.impactMode
						end,
						get = function()
							return mBI.db.profile.values.impactPerctEnabled
						end,
						set = function(info, value)
							mBI.db.profile.values.impactPerctEnabled = value
							if not value and not mBI.db.profile.iconFrame.impactIcon then
								mBI.db.profile.settings.impactMode = value
							end
						end,
					},
					impactIconEnable = {
						type = "toggle",
						name = L["Icon"],
						desc = L["Enables icon for Impact notification."],
						order = 14,
						disabled = function()
							return not mBI.db.profile.settings.impactMode or not mBI.db.profile.iconFrame.dynamic or not mBI.db.profile.iconFrame.enabled
						end,
						get = function()
							return mBI.db.profile.iconFrame.impactIcon
						end,
						set = function(info, value)
							mBI.db.profile.iconFrame.impactIcon = value
							if not value and not mBI.db.profile.values.impactPerctEnabled then
								mBI.db.profile.settings.impactMode = value
							end
						end,
					},
					priorityHeader = {
						type = "header",
						name = L["Informative icon"],
						order = 20,
					},
					dynaIcon = {
						type = "toggle",
						name = L["Dynamic icon"],
						desc = L["prioDesc"],
						order = 21,
						width = "full",
						disabled = function()
							return not mBI.db.profile.iconFrame.enabled
						end,
						get = function()
							return mBI.db.profile.iconFrame.dynamic
						end,
						set = function(info, value)
							mBI.db.profile.iconFrame.dynamic = value
							if value then
								mBI.F.icon.tt:SetTexture(mBI.lbIcon)
							else
								mBI.F.icon.tt:SetTexture(mBI.igniteIcon)
								mBI.db.profile.iconFrame.impactIcon = value
							end
						end,
					},
					space1 = {
						type = "description",
						name = " ",
						width = "full",
						order = 30,
					},
					prioFiller = {
						type = "select",
						name = L["Your filler spell"],
						order = 31,
						disabled = function()
							return not mBI.db.profile.iconFrame.enabled or not mBI.db.profile.iconFrame.dynamic
						end,
						values = {
							["fb"] = L["Fireball"],
							["ffb"] = L["ffbolt"],
						},
						get = function()
							return mBI.db.profile.iconFrame.filler
						end,
						set = function(info, value)
							mBI.db.profile.iconFrame.filler = value
						end,
					},
				},
			},
			tabReac = {
				type = "group",
				name = L["Notification"],
				desc = L["Sets how display will be adjusted upon Ignite tick value."],
				order = 6,
				args = {},
			},
			-- tabHUD = {
				-- type = "group",
				-- name = L["HUD"],
				-- desc = L["Heads up display settings."],
				-- order = 7,
				-- disabled = true,
				-- args = {},
			-- },
			tabAdvanced = {
				type = "group",
				name = L["Advanced"],
				desc = L["Advanced stuff, be carefull!"],
				order = 8,
				args = {
					header1 = {
						type = "header",
						order = 1,
						name = L["Red Zone"],
					},
					redZone = {
						type = "range",
						name = L["Red Zone"],
						desc = L["Time before expiration of one of the DoTs that will put myBigIgnite display in the low Ignite tick state."],
						order = 2,
						min = 0.1,
						max = 1,
						softMin = 0.25,
						softMax = 0.75,
						step = 0.05,
						bigStep = 0.05,
						get = function()
							return mBI.db.profile.settings.lag
						end,
						set = function(info, value)
							mBI.db.profile.settings.lag = value
						end,
					},
					redZoneVisual = {
						type = "toggle",
						name = L["Visual red zone"],
						desc = L["Display will be of the choosen colour under the red zone time."],
						order = 3,
						get = function()
							return mBI.db.profile.settings.lagVisual
						end,
						set = function(info, value)
							mBI.db.profile.settings.lagVisual = value
						end,
					},
					redZoneColor = {
						type = "color",
						name = L["Red zone colour"],
						order = 4,
						disabled = function()
							return not mBI.db.profile.settings.lagVisual
						end,
						get = function()
							return getColor(mBI.db.profile.settings.lagColor)
						end,
						set = function(info, r, g ,b)
							setColor(mBI.db.profile.settings.lagColor, r, g, b)
						end,
					},
					redZoneType = {
						type = "multiselect",
						name = L["Red zone type"],
						desc= L["Selects how red zone is displayed."],
						order = 5,
						disabled = function()
							return not mBI.db.profile.settings.lagVisual
						end,
						values = {
							text = L["Text"],
							bg = L["Background"],
							bd = L["Border"],
						},
						get = function(info, key)
							return mBI.db.profile.settings.lagVisualType[key]
						end,
						set = function(info, key, value)
							mBI.db.profile.settings.lagVisualType[key] = value
						end,							
					},					
					space1 = {
						type = "description",
						name = " ",
						width = "full",
						order = 6,
					},
					igniteTimer = {
						type = "group",
						inline = true,
						order = 7,
						args = {
							header1 = {
								type = "header",
								order = 1,
								name = L["Ignite timer"],
							},
							enabled = {
								type = "toggle",
								order = 3,
								name = L["Enable"],
								desc = L["Activates a timer bar showing visually the remaining time of an Ignite dot."],
								get = function()
									return mBI.db.profile.igniteTimer.enabled
								end,
								set = function(info, value)
									mBI.db.profile.igniteTimer.enabled = value
									if mBI.db.profile.igniteTimer.enabled then
										if not mBI.F.igniteTimer then
											mBI.F.igniteTimer = CreateFrame("StatusBar", nil, mBI.F.main)	
										end
										self:SetIgniteTimer(mBI.F.igniteTimer)
									end
									if value then
										mBI.F.igniteTimer:Show()
									else
										mBI.F.igniteTimer:Hide()
									end
								end,
							},
							space1 = {
								type = "description",
								name = " ",
								width = "full",
								order = 5,
							},
							reverse = {
								type = "toggle",
								order = 7,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled
								end,
								name = L["Reversed"],
								get = function()
									return mBI.db.profile.igniteTimer.reverse
								end,
								set = function(info, value)
									mBI.db.profile.igniteTimer.reverse = value
									self:SetIgniteTimer(mBI.F.igniteTimer)
								end,
							},
							useGeneralBackground = {
								type = "toggle",
								order = 9,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled
								end,
								name = L["Background texture"],
								desc = L["myBigIgnite background texture is applied to the bar."],
								get = function()
									return mBI.db.profile.igniteTimer.useGeneralBackground
								end,
								set = function(info, value)
									mBI.db.profile.igniteTimer.useGeneralBackground = value
									self:SetIgniteTimer(mBI.F.igniteTimer)
								end,
							},
							texture = {
								type = "select",
								order = 11,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled or mBI.db.profile.igniteTimer.useGeneralBackground
								end,
								name = L["Texture"],
								values = self:List("statusbar"),
								get = function()
									return mBI.db.profile.igniteTimer.texture
								end,
								set = function(info, value)
									mBI.db.profile.igniteTimer.texture = value
									self:SetIgniteTimer(mBI.F.igniteTimer)
								end,
							},
							space2 = {
								type = "description",
								name = " ",
								width = "full",
								order = 12,
							},
							sizeMult = {
								type = "range",
								order = 13,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled
								end,
								name = L["Size multiplier"],
								desc = L["Controls the size of the bar."],
								isPercent = true,
								min = 0.5, max = 2, bigStep = 0.05,
								get = function()
									return mBI.db.profile.igniteTimer.mult
								end,
								set = function(info, value)
									mBI.db.profile.igniteTimer.mult = value
									self:SetIgniteTimer(mBI.F.igniteTimer)
								end,
							},
							ofx = {
								type = "range",
								order = 14,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled
								end,
								name = L["Horizontal offset"],
								min = -20, max = 20, bigStep = 0.05,
								get = function()
									return mBI.db.profile.igniteTimer.ofx
								end,
								set = function(info, value)
									mBI.db.profile.igniteTimer.ofx = value
									self:SetIgniteTimer(mBI.F.igniteTimer)
								end,
							},
							ofy = {
								type = "range",
								order = 15,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled
								end,
								name = L["Vertical offset"],
								min = -20, max = 20, bigStep = 0.05,
								get = function()
									return mBI.db.profile.igniteTimer.ofy
								end,
								set = function(info, value)
									mBI.db.profile.igniteTimer.ofy = value
									self:SetIgniteTimer(mBI.F.igniteTimer)
								end,
							},
							space3 = {
								type = "description",
								name = " ",
								width = "full",
								order = 16,
							},
							alpha = {
								type = "range",
								order = 17,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled
								end,
								name = L["Alpha"],
								min = 0, max = 1, bigStep = 0.05,
								get = function()
									return mBI.db.profile.igniteTimer.alpha
								end,
								set = function(info, value)
									mBI.db.profile.igniteTimer.alpha = value
								end,
							},
							hasSpeColor = {
								type = "toggle",
								order = 18,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled
								end,
								name = L["Toggle colour"],
								desc = L["Allow the bar to has its own colour."],
								get = function()
									return mBI.db.profile.igniteTimer.specificColor.enabled
								end,
								set = function(info, value)
									mBI.db.profile.igniteTimer.specificColor.enabled = value
								end,
							},
							speColor = {
								type = "color",
								order = 19,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled or not mBI.db.profile.igniteTimer.specificColor.enabled
								end,
								name = L["Specific colour"],
								get = function()
									return getColor(mBI.db.profile.igniteTimer.specificColor.color)
								end,
								set = function(info, r, g, b)
									setColor(mBI.db.profile.igniteTimer.specificColor.color, r, g, b)
								end,
							},
							space4 = {
								type = "description",
								name = " ",
								width = "full",
								order = 20,
							},
							redZone = {
								type = "range",
								order = 21,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled
								end,
								name = L["Ignite red zone"],
								desc = L["The timer bar will turn of the selected colour when the remaining time of Ignite is under this value."],
								min = 0, max = 2, step = 0.05, bigStep = 0.1,
								get = function()
									return mBI.db.profile.igniteTimer.redZone
								end,
								set = function(info, value)
									mBI.db.profile.igniteTimer.redZone = value
								end,
							},
							rzColor = {
								type = "color",
								order = 23,
								disabled = function()
									return not mBI.db.profile.igniteTimer.enabled
								end,
								name = L["Red zone colour"],
								get = function()
									return getColor(mBI.db.profile.igniteTimer.redZoneColor)
								end,
								set = function(info, r, g, b)
									setColor(mBI.db.profile.igniteTimer.redZoneColor, r, g, b)
								end,
							},
							space5 = {
								type = "description",
								name = " ",
								width = "full",
								order = 25,
							},
						},
					},
					header2 = {
						type = "header",
						order = 8,
						name = L["Advanced settings"],
					},
					refresh = {
						type = "range",
						name = L["Refresh"],
						desc = L["Better to know what you're doing..."],
						order = 9,
						min = 0.1,
						max = 1,
						softMin = 0.1,
						softMax = 1,
						step = 0.05,
						bigStep = 0.05,
						get = function()
							return mBI.db.profile.settings.refresh
						end,
						set = function(info, value)
							mBI.db.profile.settings.refresh = value
						end,
					},
					refreshCM = {
						type = "range",
						name = L["C.M. refresh"],
						desc = L["Config mode refresh speed."],
						order = 10,
						min = 0.5,
						max = 3,
						softMin = 0.5,
						softMax = 2,
						step = 0.1,
						bigStep = 0.1,
						get = function()
							return mBI.db.profile.settings.refreshCM
						end,
						set = function(info, value)
							mBI.db.profile.settings.refreshCM = value
						end,
					},
					space2 = {
						type = "description",
						name = " ",
						width = "full",
						order = 1,
					},
					splashMsg = {
						type = "toggle",
						name = L["Splash message"],
						desc = L["Enables the welcome message on myBigIgnite loading."],
						order = 12,
						get = function()
							return mBI.db.profile.settings.splashMsg
						end,
						set = function(info, value)
							mBI.db.profile.settings.splashMsg = value
						end,
					},
				},
			},	
		},
	}

	self:TemplateOptions()
	self:WarningOptions()
	self:PriorityOptions()
end

function myBigIgnite:PriorityOptions()
	local args = {}
	local index = {
		[1] = "livingBomb",
		[2] = "ffbolt",
		[3] = "pyroblast",
		[4] = "ignite",
	}
	
	for i = 1, #index do
		args["prio" .. index[i]] = {
			type = "select",
			name = L[index[i]],
			order = 24 + i,
			disabled = function()
				return not mBI.db.profile.iconFrame.enabled or not mBI.db.profile.spellTracking[index[i]].enabled or not mBI.db.profile.iconFrame.dynamic
			end,
			values = function()
				local t = {}
					for j = 1, #index do
						t[j] = j
					end
				return t
			end,				
			get = function()
				return mBI.db.profile.spellTracking[index[i]].order
			end,
			set = function(info, value)
				local tempOrder = mBI.db.profile.spellTracking[index[i]].order
				mBI.db.profile.spellTracking[index[i]].order = value
				
				for k, v in pairs(mBI.db.profile.spellTracking) do
					if k ~= index[i] then
						if mBI.db.profile.spellTracking[k].order == value then
							mBI.db.profile.spellTracking[k].order = tempOrder
						end
					end
				end
			end,
		}
		tinsert(mBI.fireOptions.args.tabSpells.args, args["prio" .. index[i]])
	end
end

function myBigIgnite:TemplateOptions()																		-- loop to add template options
	local args = {}
	local tempType = {
		[1] = "default",
		[2] = "original",
		[3] = "modern",
		[4] = "classic",
		[5] = "ancient",
		[6] = "simple",
	}
	for i = 1, #tempType do
		args[tempType[i]] = {
			type = "execute",
			name = L[tempType[i]],
			desc = L["Applies template: "] .. L[tempType[i]],
			image = "Interface\\Addons\\myBigIgnite\\buttons\\" .. tempType[i] .. ".tga",
			imageWidth = 128,
			imageHeight = 64,
			order = i,
			confirm = true,
			confirmText = L["This will change the current layout settings. Are you sure?"],
			func = function()
				self:ApplyTemplate(tempType[i])
			end,
		}
	end
	
	wipe(mBI.fireOptions.args.tabTemplates.args)
	mBI.fireOptions.args.tabTemplates.args = deepCopy(args)
end

function myBigIgnite:WarningOptions()																		-- loop to add warning options
	local optimalIgTable = {
		igThresholdDesc = {
			type = "description",
			name = mBI.cC .. L["Minimal value above which myBigIgnite will inform you that the last Ignite tick was optimal."],
			order = 2,
			width = "full",
			fontSize = "medium",
		},
		igThreshold = {
			type = "input",
			name = L["Ignite threshold value"],
			order = 3,
			validate = isPositiveNum,
			get = function()
				return tostring(mBI.db.profile.values.igThreshold)
			end,
			set = function(info, value)
				mBI.db.profile.values.igThreshold = tonumber(value)
				if mBI.warning then
					mBI.warning = false
				end
			end,
		},
		spaceHighIg = {
			type = "description",
			name = " ",
			widh = "full",
			order = 4,
		},
	}
	local midIgTable = {
		igPerctDesc = {
			type = "description",
			name = mBI.cC .. L["Intermediate value, under optimal value. Useful in situations where you cannot expect to get a big Ignite."],
			order = 22,
			width = "full",
			fontSize = "medium",
		},
		igPerctEnable = {
			type = "toggle",
			name = L["Intermediate threshold"],
			desc = L["Enable intermediate threshold"],
			order = 23,
			get = function()
				return mBI.db.profile.values.igPerctEnabled
			end,
			set = function(info, value)
				mBI.db.profile.values.igPerctEnabled = value
			end,
		},
		igPerctAbs = {
			type = "input",
			name = " ",
			order = 24,
			width = "half",
			disabled = true,
			get = function()
				if mBI.db.profile.values.igPerctEnabled then
					return "|cffFFFFFF" .. tostring(mBI.db.profile.values.igPerct * mBI.db.profile.values.igThreshold)
				end
			end,
		},				
		igPerct = {
			type = "range",
			name = L["% of optimal Ignite"],
			order = 25,
			disabled = function()
				return not mBI.db.profile.values.igPerctEnabled
			end,
			min = 0.05,
			max = 0.99,
			softMin = 0.5,
			softMax = 0.95,
			step = 0.05,
			isPercent = true,
			get = function()
				return mBI.db.profile.values.igPerct
			end,
			set = function (info, value)
				mBI.db.profile.values.igPerct = value
			end,
		},
		spaceMidIg = {
			type = "description",
			name = " ",
			widh = "full",
			order = 26,
		},
	}
	local impactIgTable = {
		impactPerctDesc = {
			type = "description",
			name = mBI.cC .. L["Threshold associated with Impact."],
			order = 62,
			width = "full",
			fontSize = "medium",
			disabled = function()
				return not mBI.db.profile.settings.impactMode
			end,
		},
		impactPerctAbs = {
			type = "input",
			name = " ",
			order = 64,
			width = "half",
			disabled = true,
			get = function()
				if mBI.db.profile.values.impactPerctEnabled then
					return "|cffFFFFFF" .. tostring(mBI.db.profile.values.impactPerct * mBI.db.profile.values.igThreshold)
				end
			end,
		},				
		impactPerct = {
			type = "range",
			name = L["% of optimal Ignite"],
			order = 65,
			disabled = function()
				return not mBI.db.profile.values.impactPerctEnabled or not mBI.db.profile.settings.impactMode
			end,
			min = 0.05,
			max = 1,
			softMin = 0.5,
			softMax = 1,
			step = 0.05,
			isPercent = true,
			get = function()
				return mBI.db.profile.values.impactPerct
			end,
			set = function (info, value)
				mBI.db.profile.values.impactPerct = value
			end,
		},
		spaceImpactIg = {
			type = "description",
			name = " ",
			widh = "full",
			order = 66,
		},
	}
	
	local function disOptions(opt, i)
		local disabled
		
		if opt == "BgColor" or opt == "TxtToBg" then
			disabled = not mBI.cfMode or not mBI.db.profile.mainFrame.bgEnabled
			if i == 2 then
				disabled = not mBI.cfMode or not mBI.db.profile.mainFrame.bgEnabled or not mBI.db.profile.values.igPerctEnabled
			elseif i == 4 then
				disabled = not mBI.cfMode or not mBI.db.profile.mainFrame.bgEnabled or not mBI.db.profile.values.impactPerctEnabled or not mBI.db.profile.settings.impactMode
			end
		elseif opt == "BdColor" or opt == "TxtToBd" then
			disabled = not mBI.cfMode or (not mBI.db.profile.mainFrame.bdEnabled and not string.find(mBI.db.profile.iconFrame.type, "bord"))
			if i == 2 then
				disabled = not mBI.cfMode or (not mBI.db.profile.mainFrame.bdEnabled and not string.find(mBI.db.profile.iconFrame.type, "bord")) or not mBI.db.profile.values.igPerctEnabled
			elseif i == 4 then
				disabled = not mBI.cfMode or (not mBI.db.profile.mainFrame.bdEnabled and not string.find(mBI.db.profile.iconFrame.type, "bord")) or not mBI.db.profile.values.impactPerctEnabled or not mBI.db.profile.settings.impactMode
			end
		elseif opt == "BgToBd" then
			disabled = not mBI.cfMode or mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObg or mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObd or not mBI.db.profile.mainFrame.bgEnabled or (not mBI.db.profile.mainFrame.bdEnabled and not string.find(mBI.db.profile.iconFrame.type, "bord"))
			if i == 2 then
				disabled = not mBI.cfMode or mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObg or mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObd or not mBI.db.profile.mainFrame.bgEnabled or (not mBI.db.profile.mainFrame.bdEnabled and not string.find(mBI.db.profile.iconFrame.type, "bord")) or not mBI.db.profile.values.igPerctEnabled
			elseif i == 4 then
				disabled = not mBI.cfMode or mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObg or mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObd or not mBI.db.profile.mainFrame.bgEnabled or (not mBI.db.profile.mainFrame.bdEnabled and not string.find(mBI.db.profile.iconFrame.type, "bord")) or not mBI.db.profile.values.impactPerctEnabled or not mBI.db.profile.settings.impactMode
			end
		else
			disabled = not mBI.cfMode
			if i == 2 then
				disabled = not mBI.cfMode or not mBI.db.profile.values.igPerctEnabled
			elseif i == 4 then
				disabled = not mBI.cfMode or not mBI.db.profile.values.impactPerctEnabled or not mBI.db.profile.settings.impactMode
			end
		end
		
		return disabled
	end
	
	local n = 1
	local args = {}
	for i = 1, #mBI.igT do				
		local reacOptions = {
			[mBI.igT[i] .. "Header"] = {
				type = "header",
				name = L["DispIgTickValue" .. i],
				width = "full",
				order = n,
				fontSize = "medium",
			},
			[mBI.igT[i] .. "Mult"] = {
				type = "range",
				name = L["Scale multiplier"],
				desc = L["Main window is enlarged by this setting when the corresponding threshold is reached."],
				order = 6 + n,
				disabled = function()
					return disOptions(nil, i)
				end,
				hidden = false,
				min = 1.1,
				max = 3,
				softMin = 1.1,
				softMax = 2,
				step = 0.1,
				bigStep = 0.1,
				get = function()
					return mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].mult
				end,
				set = function(info, value)
					mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].mult = value
				end,
			},
			["space" .. n] = {
				type = "description",
				name = " ",
				width = "full",
				order = 7 + n,
			},
			[mBI.igT[i] .. "TextColor"] = {
				type = "color",
				name = L["Text colour"],
				order = 8 + n,
				hasAlpha = true,
				disabled = function()
					return disOptions(nil, i)
				end,
				get = function()
					return getColor(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].textColor)
				end,
				set = function (info, r, g, b, a)
					if mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObg then
						setColor(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bgColor, r, g, b)
					end
					if mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObd then
						setColor(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bdColor, r, g, b)
					end
					setColor(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].textColor, r, g, b, a)
					self:CreateColorTables()
				end,
			},					
			[mBI.igT[i] .. "BgColor"] = {
				type = "color",
				name = L["Background colour"],
				order = 9 + n,
				hasAlpha = true,
				disabled = function()
					return disOptions("BgColor", i)
				end,
				get = function()
					return getColor(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bgColor)
				end,
				set = function(info, r, g, b, a)
					if mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bgTObd then
						setColor(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bdColor, r, g, b)
					end
					if mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObg then
						mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bgColor.a = a
					else
						setColor(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bgColor, r, g, b, a)
					end
					self:CreateColorTables()
				end,
			},
			[mBI.igT[i] .. "BdColor"] = {
				type = "color",
				name = L["Border colour"],
				order = 10 + n,
				hasAlpha = true,
				disabled = function()
					return disOptions("BdColor", i)
				end,
				get = function()
					return getColor(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bdColor)
				end,
				set = function (info, r, g, b, a)
					if mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObd or mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bgTObd then
						mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bdColor.a = a
					else
						setColor(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bdColor, r, g, b, a)
					end
					self:CreateColorTables()
				end,
			},
			[mBI.igT[i] .. "TxtToBg"] = {
				type = "toggle",
				name = L["Text to Background"],
				desc = L["Applies the colour of the text to the background. Transparency is not affected."],
				order = 11 + n,
				disabled = function()
					return disOptions("TxtToBg", i)
				end,
				get = function()
					return mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObg
				end,
				set = function (info, value)
					setLink(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].textColor, mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bgColor, value)
					mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObg = value
					self:CreateColorTables()
				end,
			},
			[mBI.igT[i] .. "TxtToBd"] = {
				type = "toggle",
				name = L["Text to border"],
				desc = L["Applies the colour of the text to the border. Transparency is not affected."],
				order = 12 + n,
				disabled = function()
					return disOptions("TxtToBd", i)
				end,
				get = function()
					return mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObd
				end,
				set = function (info, value)
					setLink(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].textColor, mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bdColor, value)
					mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].txtTObd = value
					self:CreateColorTables()
				end,
			},
			[mBI.igT[i] .. "BgToBd"] = {
				type = "toggle",
				name = L["Background to border"],
				desc = L["Applies the colour of the background to the border. Transparency is not affected."],
				order = 13 + n,
				disabled = function()
					return disOptions("BgToBd", i)
				end,
				get = function()
					return mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bgTObd
				end,
				set = function (info, value)
					setLink(mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bgColor, mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bdColor, value)
					mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].bgTObd = value
					self:CreateColorTables()
				end,
			},
			[mBI.igT[i] .. "FontDecoration"] = {
				type = "select",
				name = L["Text outline"],
				order = 15 + n,
				disabled = function()
					return disOptions(nil, i)
				end,
				values = {
					NONE = L["None"],
					OUTLINE = L["Outline"],
					THICKOUTLINE = L["Thickoutline"],
				},
				get = function()
					if mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].textFlag == "" then
						mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].textFlag = "NONE"
					end
					return mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].textFlag
				end,
				set = function(info, value)
					if value == "NONE" then
						value = ""
					end
					mBI.db.profile.mainFrame[mBI.igT[i] .. "Ig"].textFlag = value
					self:SetFrameSize("dmg")
					if mBI.db.profile.titleFrame.enabled then
						self:SetFrameSize("title")
					end
					if mBI.db.profile.iconFrame.enabled then
						self:SetFrameSize("icon")
						self:SetFramePoint("icon", mBI.db.profile.iconFrame.side, mBI.F.main, mBI.db.profile.iconFrame.side, mBI.F.icon.off, 0)
					end
					self:SetFrameSize("main")
				end,
			},
			["space" .. 1 + n] = {
				type = "description",
				name = " ",
				width = "full",
				order = 16 + n,
			},
		}
		if i == 3 then
			reacOptions[mBI.igT[i] .. "Mult"].hidden = true
			for k, v in pairs(optimalIgTable) do
				reacOptions[k] = v
			end
		end			
		if i == 2 then
			for k, v in pairs(midIgTable) do
				reacOptions[k] = v
			end
		end
		if i == 4 then
			for k, v in pairs(impactIgTable) do
				reacOptions[k] = v
			end
		end
		for k, v in pairs(reacOptions) do
			args[k] = v
		end
		n = n + 20
	end
	wipe(mBI.fireOptions.args.tabReac.args)
	mBI.fireOptions.args.tabReac.args = deepCopy(args)
end


-------------------------------------------------------------------------------------------------------------------------------------------
-- templates

function myBigIgnite:Templates()
	mBI.templates = {
		modern = {
			mainFrame = {
				align = "RIGHT",
				textSize = 11,
				textFont = "Friz Quadrata TT",
				textAlign = "RIGHT",
				textOffsetX = 2,
				lowIg = {
					bgColor = {
						a = 0.5,
					},
				},
				midIg = {
					bgColor = {
						a = 0.5,
					},
				},
				highIg = {
					bgColor = {
						a = 0.5,
					},
				},
				impactIg = {
					bgColor = {
						a = 0.5,
					},
				},
			},
			iconFrame = {
				enabled = true,
				sizeFactor = 1.35,
				type = "zoombord",
			},
		},
		original = {
			mainFrame = {
				align = "RIGHT",
				textFont = "Friz Quadrata TT",
				bgEnabled = false,
				bdEnabled = false,
				textAlign = "RIGHT",
				lowIg = {
					textFlag = "OUTLINE",
				},
				midIg = {
					textFlag = "OUTLINE",
				},
				highIg = {
					textFlag = "OUTLINE",
				},
				impactIg = {
					textFlag = "OUTLINE",
				},				
			},
		},
		classic = {
			mainFrame = {
				textSize = 14,
				textFont = "Friz Quadrata TT",
				bdTexture = "Blizzard Tooltip",
				bdSize = 12,
				inset = 3.5,
				textOffsetX = 3.8,
				textOffsetY = 1.5,
				lowIg = {
					bgColor = {
						r = 0.047,
						g = 0.11,
						b = 0.227,
						a = 0.83,
					},
					bdColor = {
						r = 1,
						g = 1,
						b = 1,
						a = 1,
					},
				},
				midIg = {
					bgColor = {
						r = 0.047,
						g = 0.11,
						b = 0.227,
						a = 0.83,
					},
					bdColor = {
						r = 1,
						g = 1,
						b = 1,
						a = 1,
					},				
				},
				highIg = {
					bgColor = {
						r = 0.047,
						g = 0.11,
						b = 0.227,
						a = 0.83,
					},
					bdColor = {
						r = 1,
						g = 1,
						b = 1,
						a = 1,
					},
				},
				impactIg = {
					bgColor = {
						r = 0.047,
						g = 0.11,
						b = 0.227,
						a = 0.83,
					},
					bdColor = {
						r = 1,
						g = 1,
						b = 1,
						a = 1,
					},
				},				
			},
			titleFrame = {
				enabled = true,
			},
		},
		default = {
		},
		ancient = {
			mainFrame = {
				textSize = 11,
				textFont = "Morpheus",
				bgTexture = "Blizzard Parchment",
				bdTexture = "Blizzard Achievement Wood",
				bdSize = 7.3,
				inset = 0.5,
				textOffsetX = 3.8,
				lowIg = {
					bdColor = {
						a = 0.5,
					},
				},
				midIg = {
					bdColor = {
						a = 0.5,
					},				
				},
				highIg = {
					bdColor = {
						a = 0.5,
					},
				},
				impactIg = {
					bdColor = {
						a = 0.5,
					},
				},
			},
			iconFrame = {
				enabled = true,
				side = "RIGHT",
				sizeFactor = 0.7,
				type = "zoomed",
			},
			titleFrame = {
				enabled = true,
			},
		},
		simple = {
			mainFrame = {
				textSize = 11,
				textFont = "Skurri",
				bgEnabled = false,
				bdEnabled = false,
				textAlign = "CENTER",
			},
			iconFrame = {
				enabled = true,
				side = "LEFT",
				sizeFactor = 0.8,
				type = "zoomed",
			},
			titleFrame = {
				enabled = true,
			},
		},
	}
end


-------------------------------------------------------------------------------------------------------------------------------------------
-- managing of config panels and templates

function myBigIgnite:OpenConfig(arg)																		-- open config function
	if arg == "config" then
		self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
		self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
		self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
		InterfaceOptionsFrame_OpenToCategory("myBigIgnite")
	else
		print(mBI.cD .. "myBigIgnite v" .. mBI.addonV .. " | Istaran - Medivh EU.")
		print(mBI.cL .. L["/mbi config or /myBigIgnite config to open configuration panel."])
	end
end

function myBigIgnite:ChangeConfigDisplay(state)																-- switch between real and fake options
	wipe(mBI.options.args)
	if state == "ooFireSpec" then
		if mBI.ooFOptions == nil then
			self:LoadooFOptions()
		end
		mBI.options.args = deepCopy(mBI.ooFOptions.args)
	elseif state == "fireSpec" then
		if mBI.fireOptions == nil then
			self:LoadFireOptions()
		end
		mBI.options.args = deepCopy(mBI.fireOptions.args)
		mBI.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(myBigIgnite.db)
	else
		mBI.options.args = deepCopy(mBI.fakeOptions.args)
	end
	ACR:NotifyChange("myBigIgnite")
end

function myBigIgnite:RefreshConfig()																		-- profiles refreshing
	self:CreateSpellTables()
	self:ResetLayout()
	self:SetAnchorFramePosition()
	self:SetAnchorFrameStrata()
end

function myBigIgnite:ApplyTemplate(template)																-- switches database settings
	self:Templates()
	wipe(mBI.db.profile.mainFrame)
	wipe(mBI.db.profile.iconFrame)
	wipe(mBI.db.profile.titleFrame)
	mBI.db.profile.mainFrame = selectCopy(mBI.templates[template].mainFrame, mBI.defaults.profile.mainFrame)
	mBI.db.profile.iconFrame = selectCopy(mBI.templates[template].iconFrame, mBI.defaults.profile.iconFrame)
	mBI.db.profile.titleFrame = selectCopy(mBI.templates[template].titleFrame, mBI.defaults.profile.titleFrame)

	self:ResetLayout()																						-- applies settings
end

function myBigIgnite:ResetLayout()																			-- resets layout
	mBI.font = LSM:Fetch("font", mBI.db.profile.mainFrame.textFont)
	self:SetFrameSize("dmg")
	self:SetFramePoint("dmg", mBI.db.profile.mainFrame.textAlign, mBI.F.main, mBI.db.profile.mainFrame.textAlign, 0, 0)
	
	if mBI.F.title then
		self:SetFrameSize("title")
		self:SetFramePoint("title", mBI.db.profile.mainFrame.textAlign, mBI.F.main, mBI.db.profile.mainFrame.textAlign, 0, 0)
		if not mBI.db.profile.titleFrame.enabled then
			mBI.F.title:Hide()
		else
			mBI.F.title:Show()
		end
	end
	if mBI.db.profile.titleFrame.enabled and not mBI.F.title then
		self:CreateTitleFrame()
	end

	if mBI.F.icon then
		self:SetFrameSize("icon")
		self:SetFramePoint("icon", mBI.db.profile.iconFrame.side, mBI.F.main, mBI.db.profile.iconFrame.side, mBI.F.icon.off, 0)
		self:SetIconType("icon", mBI.db.profile.iconFrame.type)
		self:SetFrameBackdrop("icon")
		if not mBI.db.profile.iconFrame.enabled then
			mBI.F.icon:Hide()
		else
			mBI.F.icon:Show()
		end
	end	
	if mBI.db.profile.iconFrame.enabled and not mBI.F.icon then
		self:CreateIconFrame()
	end
	
	self:SetFrameSize("main")
	self:SetFramePoint("main", mBI.db.profile.mainFrame.align, mBI.F.anchor, "CENTER")
	self:SetFrameBackdrop("main")
	mBI.F.main:Show()
	self:CreateColorTables()
	self:UpdateDisplay("low")
	if not mBI.cfMode and mBI.db.profile.mainFrame.hiding.noCombat then
		self:FrameFading(true)
	end
end


-------------------------------------------------------------------------------------------------------------------------------------------
-- Config mode functions

function myBigIgnite:ConfigMode(cfmode)																		-- config mode
	if cfmode then
		mBI.F.anchor:SetMovable(true)
		mBI.F.main:EnableMouse(true)
		mBI.F.main:Show()
		if mBI.db.profile.igniteTimer.enabled then
			mBI.F.igniteTimer:SetMinMaxValues(0, 1)
			mBI.F.igniteTimer:SetValue(1)
			mBI.F.igniteTimer:Show()
		end
		mBI.F.main:SetScript("OnMouseDown", function(self, button)
			mBI.F.anchor.isMoving = true
			mBI.F.anchor:StartMoving()
		end)
		mBI.F.main:SetScript("OnMouseUp", function(self, button) 
				if mBI.F.anchor.isMoving then
					local UIScale = UIParent:GetScale()
					mBI.F.anchor.isMoving = nil
					mBI.F.anchor:StopMovingOrSizing()
					mBI.db.profile.anchorFrame.coord.xcd, mBI.db.profile.anchorFrame.coord.ycd = mBI.F.anchor:GetLeft() * UIScale, mBI.F.anchor:GetBottom() * UIScale
				end
		end)
		
		mBI.F.config = mBI.F.config or CreateFrame("Frame", nil, mBI.F.anchor)
		mBI.F.config:SetMovable(false)
		mBI.F.config:EnableMouse(false)
		mBI.F.config:Show()
		local i = 0
		mBI.F.config:SetScript("OnUpdate", function(self, elapsed)
			self.TimeSinceLastUpdate = (self.TimeSinceLastUpdate or 0) + elapsed
			if mBI.inCombat then
				mBI.cfMode = false
				myBigIgnite:ConfigMode(mBI.cfMode)
			end
			if (self.TimeSinceLastUpdate > mBI.db.profile.settings.refreshCM) then
				i = i + 1
				self.TimeSinceLastUpdate = 0
			end
			if i == 0 then 
				myBigIgnite:UpdateDisplay("low")
				myBigIgnite:UpdateIgniteTimerDisplay("low", mBI.db.profile.igniteTimer.redZone + 0.1)
				mBI.F.dmg.txt:SetText(formatText(0))
			elseif i == 1 then
				if mBI.db.profile.values.igPerctEnabled then
					myBigIgnite:UpdateDisplay("mid")
					myBigIgnite:UpdateIgniteTimerDisplay("mid", mBI.db.profile.igniteTimer.redZone + 0.1)
					mBI.F.dmg.txt:SetText(formatText(mBI.db.profile.values.igThreshold * mBI.db.profile.values.igPerct))
				end
			elseif i == 2 then
				myBigIgnite:UpdateDisplay("high")
				myBigIgnite:UpdateIgniteTimerDisplay("high", mBI.db.profile.igniteTimer.redZone + 0.1)
				mBI.F.dmg.txt:SetText(formatText(mBI.db.profile.values.igThreshold))
			elseif i == 3 and mBI.db.profile.settings.impactMode then
				if mBI.db.profile.values.impactPerctEnabled then
					myBigIgnite:UpdateDisplay("impact")
					myBigIgnite:UpdateIgniteTimerDisplay("impact", mBI.db.profile.igniteTimer.redZone + 0.1)
					mBI.F.dmg.txt:SetText(formatText(mBI.db.profile.values.igThreshold * mBI.db.profile.values.impactPerct))
				end
			else
				i = 0
			end
		end)
	else
		mBI.F.dmg.txt:SetText(formatText(mBI.igDmg))
		mBI.F.anchor:SetMovable(false)
		mBI.F.main:EnableMouse(false)
		mBI.F.config:Hide()
		if mBI.db.profile.igniteTimer.enabled then
			mBI.F.igniteTimer:Hide()
		end
		self:UpdateDisplay("low")
		if mBI.db.profile.mainFrame.hiding.noCombat then
			self:FrameFading(true)
		end
	end
end


